@@ -19,6 +19,9 @@
  *
  */
 
+#define _GNU_SOURCE
+#include <getopt.h>
+
 #include <errno.h>
 #include <locale.h>
 #include <stdio.h>
@@ -38,18 +41,22 @@
 #include <X11/extensions/shape.h>
 #include <X11/extensions/Xrandr.h>
 
+#include "config.h"
 #include "awesome.h"
 #include "event.h"
 #include "layout.h"
 #include "screen.h"
-#include "util.h"
 #include "statusbar.h"
 #include "uicb.h"
 #include "window.h"
 #include "client.h"
 #include "focus.h"
 #include "ewmh.h"
-#include "awesome-client.h"
+#include "tag.h"
+#include "common/socket.h"
+#include "common/util.h"
+#include "common/version.h"
+#include "common/configopts.h"
 
 static int (*xerrorxlib) (Display *, XErrorEvent *);
 static Bool running = True;
@@ -70,7 +77,6 @@ scan()
     {
         if(XQueryTree(globalconf.display, RootWindow(globalconf.display, screen), &d1, &d2, &wins, &num))
         {
-            real_screen = screen;
             for(i = 0; i < num; i++)
             {
                 /* XGetWindowAttributes return 1 on success */
@@ -80,8 +86,7 @@ scan()
                     continue;
                 if(wa.map_state == IsViewable || window_getstate(wins[i]) == IconicState)
                 {
-                    if(screen == 0)
-                        real_screen = get_screen_bycoord(wa.x, wa.y);
+                    real_screen = screen_get_bycoord(screen, wa.x, wa.y);
                     client_manage(wins[i], &wa, real_screen);
                 }
             }
@@ -93,8 +98,7 @@ scan()
                 if(XGetTransientForHint(globalconf.display, wins[i], &d1)
                    && (wa.map_state == IsViewable || window_getstate(wins[i]) == IconicState))
                 {
-                    if(screen == 0)
-                        real_screen = get_screen_bycoord(wa.x, wa.y);
+                    real_screen = screen_get_bycoord(screen, wa.x, wa.y);
                     client_manage(wins[i], &wa, real_screen);
                 }
             }
@@ -104,41 +108,6 @@ scan()
     }
 }
 
-/** Setup everything before running
- * \param screen Screen number
- * \todo clean things...
- */
-static void
-setup(int screen)
-{
-    XSetWindowAttributes wa;
-    Statusbar *statusbar;
-    int phys_screen = get_phys_screen(screen);
-
-    /* init cursors */
-    globalconf.cursor[CurNormal] = XCreateFontCursor(globalconf.display, XC_left_ptr);
-    globalconf.cursor[CurResize] = XCreateFontCursor(globalconf.display, XC_sizing);
-    globalconf.cursor[CurMove] = XCreateFontCursor(globalconf.display, XC_fleur);
-
-    /* select for events */
-    wa.event_mask = SubstructureRedirectMask | SubstructureNotifyMask
-        | EnterWindowMask | LeaveWindowMask | StructureNotifyMask;
-    wa.cursor = globalconf.cursor[CurNormal];
-
-    XChangeWindowAttributes(globalconf.display,
-                            RootWindow(globalconf.display, phys_screen),
-                            CWEventMask | CWCursor, &wa);
-
-    XSelectInput(globalconf.display,
-                 RootWindow(globalconf.display, phys_screen),
-                 wa.event_mask);
-
-    grabkeys(phys_screen);
-
-    for(statusbar = globalconf.screens[screen].statusbar; statusbar; statusbar = statusbar->next)
-        statusbar_init(statusbar, screen);
-}
-
 /** Startup Error handler to check if another window manager
  * is already running.
  * \param disp Display
@@ -168,23 +137,21 @@ exit_on_signal(int sig __attribute__ ((unused)))
     running = False;
 }
 
-/* There's no way to check accesses to destroyed windows, thus those cases are
+/** \brief awesome xerror function
+ * There's no way to check accesses to destroyed windows, thus those cases are
  * ignored (especially on UnmapNotify's).  Other types of errors call Xlibs
  * default error handler, which may call exit.
+ * \param edpy display ref
+ * \param ee XErrorEvent event
+ * \return 0 if no error, or xerror's xlib return status
  */
 static int
-xerror(Display * edpy, XErrorEvent * ee)
+xerror(Display *edpy, XErrorEvent *ee)
 {
     if(ee->error_code == BadWindow
-       || (ee->request_code == X_SetInputFocus && ee->error_code == BadMatch)
-       || (ee->request_code == X_PolyText8 && ee->error_code == BadDrawable)
-       || (ee->request_code == X_PolyFillRectangle
-           && ee->error_code == BadDrawable)
-       || (ee->request_code == X_PolySegment && ee->error_code == BadDrawable)
-       || (ee->request_code == X_ConfigureWindow
-           && ee->error_code == BadMatch) || (ee->request_code == X_GrabKey
-                                              && ee->error_code == BadAccess)
-       || (ee->request_code == X_CopyArea && ee->error_code == BadDrawable))
+       || (ee->error_code == BadMatch && ee->request_code == X_SetInputFocus)
+       || (ee->error_code == BadValue && ee->request_code == X_KillClient)
+       || (ee->request_code == X_ConfigureWindow && ee->error_code == BadMatch))
         return 0;
     warn("fatal error: request code=%d, error code=%d\n", ee->request_code, ee->error_code);
     return xerrorxlib(edpy, ee);        /* may call exit */
@@ -195,8 +162,13 @@ xerror(Display * edpy, XErrorEvent * ee)
 static void __attribute__ ((noreturn))
 exit_help(int exit_code)
 {
-    FILE *outfile = (exit_code == EXIT_SUCCESS)?stdout:stderr;
-    fprintf(outfile, "Usage: awesome [-v | -h | -c configfile]\n");
+    FILE *outfile = (exit_code == EXIT_SUCCESS) ? stdout : stderr;
+    fprintf(outfile,
+"Usage: awesome [OPTION]\n\
+  -h, --help             show help\n\
+  -v, --version          show version\n\
+  -c, --config FILE      configuration file to use\n\
+  -k, --check            check configuration file syntax\n");
     exit(exit_code);
 }
 
@@ -211,40 +183,59 @@ main(int argc, char *argv[])
 {
     char buf[1024];
     const char *confpath = NULL;
-    int r, xfd, e_dummy, csfd;
+    int r, xfd, e_dummy, csfd, shape_event, randr_event_base, i, screen, opt;
+    ssize_t cmdlen = 1;
+    Statusbar *statusbar;
     fd_set rd;
     XEvent ev;
     Display * dpy;
-    int shape_event, randr_event_base;
-    int screen, screen_count;
-    int i, cmdlen;
     event_handler **handler;
     struct sockaddr_un *addr;
-    int args_ok = 1;
+    Client *c;
+    XSetWindowAttributes wa;
+    static struct option long_options[] =
+    {
+        {"help",    0, NULL, 'h'},
+        {"version", 0, NULL, 'v'},
+        {"check",   0, NULL, 'k'},
+        {"config",  0, NULL, 'c'},
+        {NULL,      0, NULL, 0}
+    };
+
+    /* save argv */
+    for(i = 0; i < argc; i++)
+        cmdlen += a_strlen(argv[i]) + 1;
+
+    globalconf.argv = p_new(char, cmdlen);
+    a_strcpy(globalconf.argv, cmdlen, argv[0]);
+
+    for(i = 1; i < argc; i++)
+    {
+        a_strcat(globalconf.argv, cmdlen, " ");
+        a_strcat(globalconf.argv, cmdlen, argv[i]);
+    }
 
     /* check args */
-    if(argc >= 2)
-    {
-        args_ok = 0;
-        if(!a_strcmp("-v", argv[1]) || !a_strcmp("--version", argv[1]))
+    while((opt = getopt_long(argc, argv, "vhkc:",
+                             long_options, NULL)) != -1)
+        switch(opt)
         {
-            printf("awesome " VERSION " (" RELEASE ")\n");
-            return EXIT_SUCCESS;
-        }
-        else if(!a_strcmp("-h", argv[1]) || !a_strcmp("--help", argv[1]))
+          case 'v':
+            eprint_version("awesome");
+            break;
+          case 'h':
             exit_help(EXIT_SUCCESS);
-        else if(!a_strcmp("-c", argv[1]))
-        {
-            if(a_strlen(argv[2]))
-                confpath = argv[2], args_ok = 1;
+            break;
+          case 'c':
+            if(a_strlen(optarg))
+                confpath = optarg;
             else
                 eprint("-c option requires a file name\n");
+            break;
+          case 'k':
+            return config_check(confpath);
+            break;
         }
-        else
-            exit_help(EXIT_FAILURE);
-    }
-    if(!args_ok)
-        exit_help(EXIT_FAILURE);
 
     /* Text won't be printed correctly otherwise */
     setlocale(LC_CTYPE, "");
@@ -253,16 +244,6 @@ main(int argc, char *argv[])
     if(!(dpy = XOpenDisplay(NULL)))
         eprint("cannot open display\n");
 
-    for(cmdlen = 0, i = 0; i < argc; i++)
-        cmdlen += a_strlen(argv[i] + 1);
-    globalconf.argv = p_new(char, cmdlen);
-    a_strcpy(globalconf.argv, cmdlen, argv[0]);
-    for(i = 1; i < argc; i++)
-    {
-        a_strcat(globalconf.argv, cmdlen, " ");
-        a_strcat(globalconf.argv, cmdlen, argv[i]);
-    }
-
     xfd = ConnectionNumber(dpy);
 
     XSetErrorHandler(xerrorstart);
@@ -283,56 +264,80 @@ main(int argc, char *argv[])
     ewmh_init_atoms();
 
     /* init screens struct */
-    screen_count = get_screen_count();
-    globalconf.screens = p_new(VirtScreen, screen_count);
-    globalconf.nscreens = screen_count;
+    screen_build_screens();
     focus_add_client(NULL);
 
     /* parse config */
     config_parse(confpath);
 
+    /* scan existing windows */
+    scan();
+
+    /* init cursors */
+    globalconf.cursor[CurNormal] = XCreateFontCursor(globalconf.display, XC_left_ptr);
+    globalconf.cursor[CurResize] = XCreateFontCursor(globalconf.display, XC_sizing);
+    globalconf.cursor[CurMove] = XCreateFontCursor(globalconf.display, XC_fleur);
+
     /* for each virtual screen */
-    for(screen = 0; screen < get_screen_count(); screen++)
-        setup(screen);
+    for(screen = 0; screen < globalconf.nscreen; screen++)
+    {
+        /* view at least one tag */
+        tag_view(globalconf.screens[screen].tags, True);
+
+        for(statusbar = globalconf.screens[screen].statusbar; statusbar; statusbar = statusbar->next)
+            statusbar_init(statusbar);
+    }
+
+    /* select for events */
+    wa.event_mask = SubstructureRedirectMask | SubstructureNotifyMask
+        | EnterWindowMask | LeaveWindowMask | StructureNotifyMask | PointerMotionMask;
+    wa.cursor = globalconf.cursor[CurNormal];
 
     /* do this only for real screen */
     for(screen = 0; screen < ScreenCount(dpy); screen++)
     {
-        loadawesomeprops(screen);
+        XChangeWindowAttributes(globalconf.display,
+                                RootWindow(globalconf.display, screen),
+                                CWEventMask | CWCursor, &wa);
+        XSelectInput(globalconf.display,
+                     RootWindow(globalconf.display, screen),
+                     wa.event_mask);
         ewmh_set_supported_hints(screen);
+        /* call this to at least grab root window clicks */
+        window_root_grabbuttons(screen);
+        window_root_grabkeys(screen);
     }
 
     handler = p_new(event_handler *, LASTEvent);
-    handler[ButtonPress] = handle_event_buttonpress;
-    handler[ConfigureRequest] = handle_event_configurerequest;
-    handler[ConfigureNotify] = handle_event_configurenotify;
-    handler[DestroyNotify] = handle_event_destroynotify;
-    handler[EnterNotify] = handle_event_enternotify;
-    handler[LeaveNotify] = handle_event_leavenotify;
-    handler[Expose] = handle_event_expose;
-    handler[KeyPress] = handle_event_keypress;
-    handler[MappingNotify] = handle_event_mappingnotify;
-    handler[MapRequest] = handle_event_maprequest;
-    handler[PropertyNotify] = handle_event_propertynotify;
-    handler[UnmapNotify] = handle_event_unmapnotify;
-    handler[ClientMessage] = handle_event_clientmessage;
+    handler[ButtonPress] = event_handle_buttonpress;
+    handler[ConfigureRequest] = event_handle_configurerequest;
+    handler[ConfigureNotify] = event_handle_configurenotify;
+    handler[DestroyNotify] = event_handle_destroynotify;
+    handler[EnterNotify] = event_handle_enternotify;
+    handler[LeaveNotify] = event_handle_leavenotify;
+    handler[MotionNotify] = event_handle_motionnotify;
+    handler[Expose] = event_handle_expose;
+    handler[KeyPress] = event_handle_keypress;
+    handler[MappingNotify] = event_handle_mappingnotify;
+    handler[MapRequest] = event_handle_maprequest;
+    handler[PropertyNotify] = event_handle_propertynotify;
+    handler[UnmapNotify] = event_handle_unmapnotify;
+    handler[ClientMessage] = event_handle_clientmessage;
 
     /* check for shape extension */
     if((globalconf.have_shape = XShapeQueryExtension(dpy, &shape_event, &e_dummy)))
     {
         p_realloc(&handler, shape_event + 1);
-        handler[shape_event] = handle_event_shape;
+        handler[shape_event] = event_handle_shape;
     }
 
     /* check for randr extension */
     if((globalconf.have_randr = XRRQueryExtension(dpy, &randr_event_base, &e_dummy)))
     {
         p_realloc(&handler, randr_event_base + RRScreenChangeNotify + 1);
-        handler[randr_event_base + RRScreenChangeNotify] = handle_event_randr_screen_change_notify;
+        handler[randr_event_base + RRScreenChangeNotify] = event_handle_randr_screen_change_notify;
     }
 
-    scan();
-
     XSync(dpy, False);
 
     /* get socket fd */
@@ -357,8 +362,9 @@ main(int argc, char *argv[])
     signal(SIGTERM, &exit_on_signal);
     signal(SIGHUP, &exit_on_signal);
 
-    /* call this to at least grab root window clicks */
-    window_grabbuttons(DefaultScreen(dpy), None, False, True); 
+    /* refresh everything before waiting events */
+    statusbar_refresh();
+    layout_refresh();
 
     /* main event loop, also reads status text from socket */
     while(running)
@@ -387,27 +393,49 @@ main(int argc, char *argv[])
                     break;
                 buf[r] = '\0';
                 parse_control(buf);
+                statusbar_refresh();
+                layout_refresh();
             }
-
+        /* two level XPending:
+         * we need to first check we have XEvent to handle
+         * and if so, we handle them all in a round.
+         * Then when we have refresh()'ed stuff so maybe new XEvent
+         * are available and select() won't tell us, so let's check
+         * with XPending() again.
+         */
         while(XPending(dpy))
         {
-            XNextEvent(dpy, &ev);
-            if(handler[ev.type])
-                handler[ev.type](&ev);       /* call handler */
+            while(XPending(dpy))
+            {
+                XNextEvent(dpy, &ev);
+                if(handler[ev.type])
+                    handler[ev.type](&ev);
+
+                /* need to resync */
+                XSync(dpy, False);
+            }
+
+            statusbar_refresh();
+            layout_refresh();
+
+            /* need to resync */
+            XSync(dpy, False);
         }
-
-        statusbar_refresh();
-
-        /* need to resync */
-        XSync(dpy, False);
     }
 
+
     if(csfd > 0 && close(csfd))
         perror("error closing UNIX domain socket");
     if(unlink(addr->sun_path))
         perror("error unlinking UNIX domain socket");
     p_delete(&addr);
 
+    /* remap all clients since some WM won't handle them otherwise */
+    for(c = globalconf.clients; c; c = c->next)
+        client_unban(c);
+
+    XSync(globalconf.display, False);
+
     XCloseDisplay(dpy);
 
     return EXIT_SUCCESS;
