@@ -27,16 +27,14 @@
 #include "client.h"
 #include "tag.h"
 #include "rules.h"
-#include "util.h"
-#include "xutil.h"
 #include "window.h"
 #include "focus.h"
 #include "ewmh.h"
 #include "screen.h"
 #include "widget.h"
+#include "xutil.h"
 #include "layouts/floating.h"
 
-
 extern AwesomeConf globalconf;
 
 /** Load windows properties, restoring client's tag
@@ -44,6 +42,7 @@ extern AwesomeConf globalconf;
  * \todo this may bug if number of tags is != than before
  * \param c Client ref
  * \param screen Screen ID
+ * \return true if client had property
  */
 static Bool
 client_loadprops(Client * c, int screen)
@@ -72,7 +71,7 @@ client_loadprops(Client * c, int screen)
                 untag_client(c, tag);
 
         if(i <= ntags && prop[i])
-            c->isfloating = prop[i] == '1';
+            client_setfloating(c, prop[i] == '1');
     }
 
     p_delete(&prop);
@@ -86,7 +85,7 @@ client_loadprops(Client * c, int screen)
  * \return True if client has WM_DELETE_WINDOW
  */
 static Bool
-isprotodel(Display *disp, Window win)
+client_isprotodel(Display *disp, Window win)
 {
     int i, n;
     Atom *protocols;
@@ -102,47 +101,13 @@ isprotodel(Display *disp, Window win)
     return ret;
 }
 
-/** Swap two client in the linked list clients
- * \param head pointer ito the client list head
- * \param c1 first client
- * \param c2 second client
- */
-static void
-client_swap(Client **head, Client *c1, Client *c2)
-{
-    Client *tmp;
-
-    tmp = c1->next;
-    c1->next = c2->next;
-    c2->next = (tmp == c2 ? c1 : tmp);
-
-    tmp = c2->prev;
-    c2->prev = c1->prev;
-    c1->prev = (tmp == c1 ? c2 : tmp );
-
-    if(c1->next)
-        c1->next->prev = c1;
-
-    if(c1->prev)
-        c1->prev->next = c1;
-
-    if(c2->next)
-        c2->next->prev = c2;
-
-    if(c2->prev)
-        c2->prev->next = c2;
-
-    if(*head == c1)
-        *head = c2;
-}
-
 /** Get a Client by its window
  * \param list Client list to look info
  * \param w Client window to find
  * \return client
  */
 Client *
-get_client_bywin(Client *list, Window w)
+client_get_bywin(Client *list, Window w)
 {
     Client *c;
 
@@ -150,14 +115,13 @@ get_client_bywin(Client *list, Window w)
     return c;
 }
 
-
 /** Get a client by its name
  * \param list Client list
  * \param name name to search
  * \return first matching client
  */
 Client *
-get_client_byname(Client *list, char *name)
+client_get_byname(Client *list, char *name)
 {
     Client *c;
 
@@ -180,84 +144,40 @@ client_updatetitle(Client *c)
     widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
 }
 
+static void
+client_unfocus(Client *c)
+{
+    if(globalconf.screens[c->screen].opacity_unfocused != -1)
+        window_settrans(c->win, globalconf.screens[c->screen].opacity_unfocused);
+    XSetWindowBorder(globalconf.display, c->win,
+                     globalconf.screens[c->screen].colors_normal[ColBorder].pixel);
+    widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
+    focus_add_client(NULL);
+}
+
 /** Ban client and unmap it
  * \param c the client
  */
 void
-client_ban(Client * c)
+client_ban(Client *c)
 {
+    if(globalconf.focus->client == c)
+        client_unfocus(c);
     XUnmapWindow(globalconf.display, c->win);
     window_setstate(c->win, IconicState);
 }
 
-
-static void
-client_attach_at_end(Client *c)
-{
-    Client *iter;
-    for(iter = globalconf.clients; iter && iter->next; iter = iter->next);
-    /* stack is empty */
-    if(!iter)
-        globalconf.clients = c;
-    else
-    {
-        c->prev = iter;
-        iter->next = c;
-    }
-}
-
-/** Attach client to the beginning of the clients stack
- * \param c the client
- */
-void
-client_attach(Client *c)
-{
-    if(globalconf.clients)
-        (globalconf.clients)->prev = c;
-    c->next = globalconf.clients;
-    globalconf.clients = c;
-}
-
-/** Detach client from clients list
- * \param c client to detach
- */
-void
-client_detach(Client *c)
-{
-    if(c->prev)
-        c->prev->next = c->next;
-    if(c->next)
-        c->next->prev = c->prev;
-    if(c == globalconf.clients)
-        globalconf.clients = c->next;
-    c->next = c->prev = NULL;
-}
-
 /** Give focus to client, or to first client if c is NULL
  * \param c client
- * \param selscreen True if current screen is selected
  * \param screen Screen ID
  */
 void
-focus(Client *c, Bool selscreen, int screen)
+client_focus(Client *c, int screen, Bool raise)
 {
     int phys_screen = get_phys_screen(screen);
 
-    /* unfocus current selected client */
-    if(globalconf.focus->client)
-    {
-        widget_invalidate_cache(globalconf.focus->client->screen, WIDGET_CACHE_CLIENTS);
-        window_grabbuttons(get_phys_screen(globalconf.focus->client->screen),
-                           globalconf.focus->client->win, False, True);
-        XSetWindowBorder(globalconf.display, globalconf.focus->client->win,
-                         globalconf.screens[screen].colors_normal[ColBorder].pixel);
-        window_settrans(globalconf.focus->client->win,
-                        globalconf.screens[screen].opacity_unfocused);
-    }
-
-
     /* if c is NULL or invisible, take next client in the focus history */
-    if((!c && selscreen) || (c && !client_isvisible(c, screen)))
+    if(!c || (c && !client_isvisible(c, screen)))
     {
         c = focus_get_current_client(screen);
         /* if c is still NULL take next client in the stack */
@@ -265,34 +185,58 @@ focus(Client *c, Bool selscreen, int screen)
             for(c = globalconf.clients; c && (c->skip || !client_isvisible(c, screen)); c = c->next);
     }
 
+    /* unfocus current selected client */
+    if(globalconf.focus->client)
+        client_unfocus(globalconf.focus->client);
+
     if(c)
     {
-        XSetWindowBorder(globalconf.display, c->win, globalconf.screens[screen].colors_selected[ColBorder].pixel);
-        window_grabbuttons(phys_screen, c->win, True, True);
-    }
-
-    if(!selscreen)
-        return;
-
-    /* save sel in focus history */
-    focus_add_client(c);
-
-    if(globalconf.focus->client)
-    {
-        widget_invalidate_cache(screen, WIDGET_CACHE_CLIENTS);
-        XSetInputFocus(globalconf.display,
-                       globalconf.focus->client->win, RevertToPointerRoot, CurrentTime);
-        for(c = globalconf.clients; c; c = c->next)
-            if(c != globalconf.focus->client)
-                window_settrans(globalconf.focus->client->win,
-                                globalconf.screens[screen].opacity_unfocused);
-        window_settrans(globalconf.focus->client->win, -1);
+        /* unban the client before focusing or it will fail */
+        client_unban(c);
+        /* save sel in focus history */
+        focus_add_client(c);
+        if(globalconf.screens[c->screen].opacity_unfocused != -1)
+            window_settrans(c->win, -1);
+        XSetWindowBorder(globalconf.display, c->win,
+                         globalconf.screens[screen].colors_selected[ColBorder].pixel);
+        XSetInputFocus(globalconf.display, c->win, RevertToPointerRoot, CurrentTime);
+        if(raise)
+        {
+            XWindowChanges wc;
+            Layout *curlay = layout_get_current(screen);
+            if(c->isfloating || curlay->arrange == layout_floating)
+                XRaiseWindow(globalconf.display, c->win);
+            else
+            {
+                Client *client;
+                wc.stack_mode = Below;
+                wc.sibling = None;
+                for(client = globalconf.clients; client; client = client->next)
+                    if(client != c && client_isvisible(client, c->screen) && client->isfloating)
+                    {
+                        XConfigureWindow(globalconf.display, client->win, CWSibling | CWStackMode, &wc);
+                        wc.sibling = client->win;
+                    }
+                XConfigureWindow(globalconf.display, c->win, CWSibling | CWStackMode, &wc);
+                wc.sibling = c->win;
+                for(client = globalconf.clients; client; client = client->next)
+                    if(client != c && IS_TILED(client, c->screen))
+                    {
+                        XConfigureWindow(globalconf.display, client->win, CWSibling | CWStackMode, &wc);
+                        wc.sibling = client->win;
+                    }
+            }
+        }
+        /* since we're dropping EnterWindow events and sometimes the window
+         * will appear under the mouse, grabbuttons */
+        window_grabbuttons(phys_screen, c->win);
     }
     else
         XSetInputFocus(globalconf.display,
                        RootWindow(globalconf.display, phys_screen),
                        RevertToPointerRoot, CurrentTime);
 
+    widget_invalidate_cache(screen, WIDGET_CACHE_CLIENTS);
     ewmh_update_net_active_window(phys_screen);
 }
 
@@ -306,76 +250,100 @@ client_manage(Window w, XWindowAttributes *wa, int screen)
 {
     Client *c, *t = NULL;
     Window trans;
-    Bool rettrans;
+    Bool rettrans, retloadprops;
     XWindowChanges wc;
-    Area area, darea;
     Tag *tag;
     Rule *rule;
+    Area screen_geom;
     int phys_screen = get_phys_screen(screen);
-
-    area = get_screen_area(screen, NULL, NULL);
+    long flags;
 
     c = p_new(Client, 1);
 
+    c->screen = screen_get_bycoord(screen, wa->x, wa->y);
+
+    screen_geom = screen_get_area(c->screen,
+                                  globalconf.screens[screen].statusbar,
+                                  &globalconf.screens[screen].padding);
+    /* Initial values */
     c->win = w;
-    c->geometry.x = c->f_geometry.x = c->m_geometry.x = wa->x;
-    c->geometry.y = c->f_geometry.y = c->m_geometry.y = wa->y;
+    c->geometry.x = c->f_geometry.x = c->m_geometry.x = MAX(wa->x, screen_geom.x);
+    c->geometry.y = c->f_geometry.y = c->m_geometry.y = MAX(wa->y, screen_geom.y);
     c->geometry.width = c->f_geometry.width = c->m_geometry.width = wa->width;
     c->geometry.height = c->f_geometry.height = c->m_geometry.height = wa->height;
     c->oldborder = wa->border_width;
     c->newcomer = True;
 
-    c->screen = get_screen_bycoord(c->geometry.x, c->geometry.y);
+    c->border = globalconf.screens[screen].borderpx;
 
-    move_client_to_screen(c, screen, True);
-
-    /* update window title */
-    client_updatetitle(c);
-
-    if(c->geometry.width == area.width && c->geometry.height == area.height)
-        c->border = wa->border_width;
-    else
-        c->border = globalconf.screens[screen].borderpx;
-
-    ewmh_check_client_hints(c);
-    /* loadprops or apply rules if no props */
-    if(!client_loadprops(c, screen))
-        tag_client_with_rules(c);
-
-    /* if window request fullscreen mode */
-    if(c->geometry.width == area.width && c->geometry.height == area.height)
-    {
-        c->geometry.x = area.x;
-        c->geometry.y = area.y;
-    }
-    else
-    {
-        darea = get_display_area(phys_screen,
-                                 globalconf.screens[screen].statusbar,
-                                 &globalconf.screens[screen].padding);
-
-        if(c->geometry.x + c->geometry.width + 2 * c->border > darea.x + darea.width)
-            c->geometry.x = c->f_geometry.x = darea.x + darea.width - c->geometry.width - 2 * c->border;
-        if(c->geometry.y + c->geometry.height + 2 * c->border > darea.y + darea.height)
-            c->geometry.y = c->f_geometry.y = darea.y + darea.height - c->geometry.height - 2 * c->border;
-        if(c->geometry.x < darea.x)
-            c->geometry.x = c->f_geometry.x = darea.x;
-        if(c->geometry.y < darea.y)
-            c->geometry.y = c->f_geometry.y = darea.y;
-    }
-
-    /* XXX if this necessary ? */
-    /* set borders */
+    /* Set windows borders */
     wc.border_width = c->border;
     XConfigureWindow(globalconf.display, w, CWBorderWidth, &wc);
     XSetWindowBorder(globalconf.display, w, globalconf.screens[screen].colors_normal[ColBorder].pixel);
     /* propagates border_width, if size doesn't change */
     window_configure(c->win, c->geometry, c->border);
 
+    /* update window title */
+    client_updatetitle(c);
+
     /* update hints */
-    client_updatesizehints(c);
+    flags = client_updatesizehints(c);
     client_updatewmhints(c);
 
+    /* Try to load props if any */
+    retloadprops = client_loadprops(c, screen);
+
+    /* Then check clients hints */
+    ewmh_check_client_hints(c);
+
+    /* Then apply rules if no props */
+    if(!retloadprops)
+    {
+        /* Get the client's rule */
+        if((rule = rule_matching_client(c)))
+        {
+            if(rule->screen != RULE_NOSCREEN)
+                move_client_to_screen(c, rule->screen, True);
+            else
+                move_client_to_screen(c, screen, True);
+            tag_client_with_rule(c, rule);
+
+            switch(rule->isfloating)
+            {
+              case Auto:
+                break;
+              case Yes:
+                client_setfloating(c, True);
+                break;
+              case No:
+                client_setfloating(c, False);
+                break;
+            }
+
+            if(rule->opacity >= 0.0f)
+                window_settrans(c->win, rule->opacity);
+        }
+        else
+            move_client_to_screen(c, screen, True);
+    }
+
+    /* check for transient and set tags like its parent,
+     * XGetTransientForHint returns 1 on success
+     */
+    if((rettrans = XGetTransientForHint(globalconf.display, w, &trans))
+       && (t = client_get_bywin(globalconf.clients, trans)))
+        for(tag = globalconf.screens[c->screen].tags; tag; tag = tag->next)
+            if(is_client_tagged(t, tag))
+                tag_client(c, tag);
+
+    /* should be floating if transsient or fixed */
+    if(!c->isfloating)
+        client_setfloating(c, rettrans || c->isfixed);
+
+    if(!(flags & (USPosition | PPosition)))
+        c->f_geometry =
+            globalconf.screens[c->screen].floating_placement(c->f_geometry, c->border, c->screen);
+
     XSelectInput(globalconf.display, w, StructureNotifyMask | PropertyChangeMask | EnterWindowMask);
 
     /* handle xshape */
@@ -385,53 +353,35 @@ client_manage(Window w, XWindowAttributes *wa, int screen)
         window_setshape(phys_screen, c->win);
     }
 
-    /* grab buttons */
-    window_grabbuttons(phys_screen, c->win, False, True);
-
-    /* check for transient and set tags like its parent,
-     * XGetTransientForHint returns 1 on success
-     */
-    if((rettrans = XGetTransientForHint(globalconf.display, w, &trans))
-       && (t = get_client_bywin(globalconf.clients, trans)))
-        for(tag = globalconf.screens[c->screen].tags; tag; tag = tag->next)
-            if(is_client_tagged(t, tag))
-                tag_client(c, tag);
-
-    /* should be floating if transsient or fixed */
-    if(!c->isfloating)
-        c->isfloating = rettrans || c->isfixed;
-
-    /* save new props */
-    client_saveprops(c);
-
     /* attach to the stack */
-    for(rule = globalconf.rules; rule; rule = rule->next)
-        if(rule->not_master && client_match_rule(c, rule))
+    if((rule = rule_matching_client(c)))
+        switch(rule->ismaster)
         {
-            client_attach_at_end(c);
+          case Yes:
+            client_list_push(&globalconf.clients, c);
+            break;
+          case No:
+            client_list_append(&globalconf.clients, c);
+            break;
+          case Auto:
+            rule = NULL;
             break;
         }
+
     if(!rule)
     {
         if(globalconf.screens[c->screen].new_become_master)
-            client_attach(c);
+            client_list_push(&globalconf.clients, c);
         else
-            client_attach_at_end(c);
+            client_list_append(&globalconf.clients, c);
     }
 
-    ewmh_update_net_client_list(phys_screen);
-
     /* some windows require this */
     XMoveResizeWindow(globalconf.display, c->win, c->geometry.x, c->geometry.y,
                       c->geometry.width, c->geometry.height);
 
-    if(globalconf.screens[c->screen].new_get_focus)
-        focus(c, True, screen);
-    
     widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
-
-    /* rearrange to display new window */
-    arrange(c->screen);
+    ewmh_update_net_client_list(phys_screen);
 }
 
 /** Resize client window
@@ -507,10 +457,7 @@ client_resize(Client *c, Area geometry, Bool sizehints)
     if(c->geometry.x != geometry.x || c->geometry.y != geometry.y
        || c->geometry.width != geometry.width || c->geometry.height != geometry.height)
     {
-        if(XineramaIsActive(globalconf.display))
-            new_screen = get_screen_bycoord(geometry.x, geometry.y);
-        else
-            new_screen = c->screen;
+        new_screen = screen_get_bycoord(c->screen, geometry.x, geometry.y);
 
         c->geometry.x = wc.x = geometry.x;
         c->geometry.y = wc.y = geometry.y;
@@ -521,7 +468,7 @@ client_resize(Client *c, Area geometry, Bool sizehints)
         /* save the floating geometry if the window is floating but not
          * maximized */
         if((c->isfloating ||
-           get_current_layout(new_screen)->arrange == layout_floating) && !c->ismax)
+           layout_get_current(new_screen)->arrange == layout_floating) && !c->ismax)
             c->f_geometry = geometry;
 
         XConfigureWindow(globalconf.display, c->win,
@@ -536,6 +483,32 @@ client_resize(Client *c, Area geometry, Bool sizehints)
     return False;
 }
 
+void
+client_setfloating(Client *c, Bool floating)
+{
+    if(c->isfloating != floating)
+    {
+        if((c->isfloating = floating))
+        {
+            client_resize(c, c->f_geometry, False);
+            XRaiseWindow(globalconf.display, c->win);
+        }
+        else
+        {
+            XLowerWindow(globalconf.display, c->win);
+            if(c->ismax)
+            {
+                c->ismax = False;
+                client_resize(c, c->m_geometry, False);
+            }
+        }
+        if(client_isvisible(c, c->screen))
+            globalconf.screens[c->screen].need_arrange = True;
+        widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
+        client_saveprops(c);
+    }
+}
+
 /** Save client properties as an X property
  * \param c client
  */
@@ -587,13 +560,15 @@ client_unmanage(Client *c)
     XConfigureWindow(globalconf.display, c->win, CWBorderWidth, &wc);  /* restore border */
 
     /* remove client everywhere */
-    client_detach(c);
+    client_list_detach(&globalconf.clients, c);
     focus_delete_client(c);
+    if(globalconf.scratch.client == c)
+        globalconf.scratch.client = NULL;
     for(tag = globalconf.screens[c->screen].tags; tag; tag = tag->next)
         untag_client(c, tag);
 
     if(globalconf.focus->client == c)
-        focus(NULL, True, c->screen);
+        client_focus(NULL, c->screen, True);
 
     XUngrabButton(globalconf.display, AnyButton, AnyModifier, c->win);
     window_setstate(c->win, WithdrawnState);
@@ -601,7 +576,6 @@ client_unmanage(Client *c)
     XSync(globalconf.display, False);
     XUngrabServer(globalconf.display);
 
-    arrange(c->screen);
     p_delete(&c);
 }
 
@@ -623,14 +597,15 @@ client_updatewmhints(Client *c)
     }
 }
 
-void
+long
 client_updatesizehints(Client *c)
 {
     long msize;
     XSizeHints size;
 
-    if(!XGetWMNormalHints(globalconf.display, c->win, &size, &msize) || !size.flags)
-        size.flags = PSize;
+    if(!XGetWMNormalHints(globalconf.display, c->win, &size, &msize))
+        return 0L;
+
     if(size.flags & PBaseSize)
     {
         c->basew = size.base_width;
@@ -685,6 +660,8 @@ client_updatesizehints(Client *c)
     if(c->maxw && c->minw && c->maxh && c->minh
        && c->maxw == c->minw && c->maxh == c->minh)
         c->isfixed = True;
+
+    return size.flags;
 }
 
 /** Returns True if a client is tagged
@@ -699,6 +676,9 @@ client_isvisible(Client *c, int screen)
     if(!c || c->screen != screen)
         return False;
 
+    if(globalconf.scratch.client == c)
+        return globalconf.scratch.isvisible;
+
     for(tag = globalconf.screens[screen].tags; tag; tag = tag->next)
         if(tag->selected && is_client_tagged(c, tag))
             return True;
@@ -754,25 +734,33 @@ uicb_client_settrans(int screen __attribute__ ((unused)), char *arg)
         window_settrans(sel->win, delta);
 }
 
-/** Swap current with next client
- * \param screen Screen ID
- * \param arg nothing
- * \ingroup ui_callback
- */
-void
-uicb_client_swapnext(int screen, char *arg __attribute__ ((unused)))
+static Client *
+client_find_prev_visible(Client *sel)
 {
-    Client *next, *sel = globalconf.focus->client;
+    Client *prev = NULL;
 
-    if(!sel)
-        return;
+    if(!sel) return NULL;
 
-    for(next = sel->next; next && !client_isvisible(next, screen); next = next->next);
-    if(next)
-    {
-        client_swap(&globalconf.clients, sel, next);
-        arrange(screen);
-    }
+    /* look for previous starting at sel */
+    for(prev = client_list_prev_cycle(&globalconf.clients, sel);
+        prev && (prev->skip || !client_isvisible(prev, sel->screen));
+        prev = client_list_prev_cycle(&globalconf.clients, prev));
+
+    return prev;
+}
+
+static Client *
+client_find_next_visible(Client *sel)
+{
+    Client *next = NULL;
+
+    if(!sel) return NULL;
+
+    for(next = sel->next; next && !client_isvisible(next, sel->screen); next = next->next);
+    if(!next)
+        for(next = globalconf.clients; next && !client_isvisible(next, sel->screen); next = next->next);
+
+    return next;
 }
 
 /** Swap current with previous client
@@ -781,18 +769,33 @@ uicb_client_swapnext(int screen, char *arg __attribute__ ((unused)))
  * \ingroup ui_callback
  */
 void
-uicb_client_swapprev(int screen, char *arg __attribute__ ((unused)))
+uicb_client_swapprev(int screen __attribute__ ((unused)),
+                     char *arg __attribute__ ((unused)))
 {
-    Client *prev, *sel = globalconf.focus->client;
+    Client *prev;
 
-    if(!sel)
-        return;
-
-    for(prev = sel->prev; prev && !client_isvisible(prev, screen); prev = prev->prev);
-    if(prev)
+    if((prev = client_find_prev_visible(globalconf.focus->client)))
     {
-        client_swap(&globalconf.clients, prev, sel);
-        arrange(screen);
+        client_list_swap(&globalconf.clients, prev, globalconf.focus->client);
+        globalconf.screens[prev->screen].need_arrange = True;
+    }
+}
+
+/** Swap current with next client
+ * \param screen Screen ID
+ * \param arg nothing
+ * \ingroup ui_callback
+ */
+void
+uicb_client_swapnext(int screen __attribute__ ((unused)),
+                     char *arg __attribute__ ((unused)))
+{
+    Client *next;
+
+    if((next = client_find_next_visible(globalconf.focus->client)))
+    {
+        client_list_swap(&globalconf.clients, globalconf.focus->client, next);
+        globalconf.screens[next->screen].need_arrange = True;
     }
 }
 
@@ -804,24 +807,22 @@ uicb_client_swapprev(int screen, char *arg __attribute__ ((unused)))
 void
 uicb_client_moveresize(int screen, char *arg)
 {
-    int ox, oy, ow, oh;
+    int ox, oy, ow, oh; /* old geometry */
     char x[8], y[8], w[8], h[8];
     int mx, my, dx, dy, nmx, nmy;
     unsigned int dui;
     Window dummy;
     Area area;
     Client *sel = globalconf.focus->client;
-    Tag **curtags = get_current_tags(screen);
+    Layout *curlay = layout_get_current(screen);
+
+    if(!sel || sel->isfixed || !arg ||
+       (curlay->arrange != layout_floating && !sel->isfloating))
+        return;
 
-    if(curtags[0]->layout->arrange != layout_floating)
-        if(!sel || !sel->isfloating || sel->isfixed || !arg)
-        {
-            p_delete(&curtags);
-            return;
-        }
-    p_delete(&curtags);
     if(sscanf(arg, "%s %s %s %s", x, y, w, h) != 4)
         return;
+
     area.x = (int) compute_new_value_from_arg(x, sel->geometry.x);
     area.y = (int) compute_new_value_from_arg(y, sel->geometry.y);
     area.width = (int) compute_new_value_from_arg(w, sel->geometry.width);
@@ -837,23 +838,29 @@ uicb_client_moveresize(int screen, char *arg)
                                         get_phys_screen(screen)),
                              &dummy, &dummy, &mx, &my, &dx, &dy, &dui);
     client_resize(sel, area, globalconf.screens[sel->screen].resize_hints);
-    if (xqp && ox <= mx && (ox + ow) >= mx && oy <= my && (oy + oh) >= my)
+    if (xqp && ox <= mx && (ox + 2 * sel->border + ow) >= mx &&
+        oy <= my && (oy + 2 * sel->border + oh) >= my)
     {
-        nmx = mx - ox + sel->geometry.width - ow - 1 < 0 ? 0 : mx - ox + sel->geometry.width - ow - 1;
-        nmy = my - oy + sel->geometry.height - oh - 1 < 0 ? 0 : my - oy + sel->geometry.height - oh - 1;
+        nmx = mx - (ox + sel->border) + sel->geometry.width - ow;
+        nmy = my - (oy + sel->border) + sel->geometry.height - oh;
+
+        if(nmx < -sel->border) /* can happen on a resize */
+            nmx = -sel->border;
+        if(nmy < -sel->border)
+            nmy = -sel->border;
+
         XWarpPointer(globalconf.display,
                      None, sel->win,
                      0, 0, 0, 0, nmx, nmy);
     }
 }
 
-
 void
 client_kill(Client *c)
 {
     XEvent ev;
 
-    if(isprotodel(globalconf.display, c->win))
+    if(client_isprotodel(globalconf.display, c->win))
     {
         ev.type = ClientMessage;
         ev.xclient.window = c->win;
@@ -884,34 +891,31 @@ uicb_client_kill(int screen __attribute__ ((unused)), char *arg __attribute__ ((
 static void
 client_maximize(Client *c, Area geometry)
 {
-    
+
     if((c->ismax = !c->ismax))
     {
         c->wasfloating = c->isfloating;
         c->m_geometry = c->geometry;
-        if(get_current_layout(c->screen)->arrange != layout_floating)
-            c->isfloating = True;
+        if(layout_get_current(c->screen)->arrange != layout_floating)
+            client_setfloating(c, True);
+        client_focus(c, c->screen, True);
         client_resize(c, geometry, False);
-        restack(c->screen);
         widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
     }
     else if(c->wasfloating)
     {
-        c->isfloating = True;
+        client_setfloating(c, True);
         client_resize(c, c->m_geometry, False);
-        restack(c->screen);
         widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
     }
-    else if(get_current_layout(c->screen)->arrange == layout_floating)
+    else if(layout_get_current(c->screen)->arrange == layout_floating)
     {
         client_resize(c, c->m_geometry, False);
-        restack(c->screen);
         widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
     }
     else
     {
-        c->isfloating = False;
-        arrange(c->screen);
+        client_setfloating(c, False);
         widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
     }
 }
@@ -925,7 +929,7 @@ void
 uicb_client_togglemax(int screen, char *arg __attribute__ ((unused)))
 {
     Client *sel = globalconf.focus->client;
-    Area area = get_screen_area(screen,
+    Area area = screen_get_area(screen,
                                 globalconf.screens[screen].statusbar,
                                 &globalconf.screens[screen].padding);
 
@@ -946,7 +950,7 @@ void
 uicb_client_toggleverticalmax(int screen, char *arg __attribute__ ((unused)))
 {
     Client *sel = globalconf.focus->client;
-    Area area = get_screen_area(screen,
+    Area area = screen_get_area(screen,
                                 globalconf.screens[screen].statusbar,
                                 &globalconf.screens[screen].padding);
 
@@ -969,7 +973,7 @@ void
 uicb_client_togglehorizontalmax(int screen, char *arg __attribute__ ((unused)))
 {
     Client *sel = globalconf.focus->client;
-    Area area = get_screen_area(screen,
+    Area area = screen_get_area(screen,
                                 globalconf.screens[screen].statusbar,
                                 &globalconf.screens[screen].padding);
 
@@ -992,16 +996,104 @@ uicb_client_zoom(int screen, char *arg __attribute__ ((unused)))
 {
     Client *c, *sel = globalconf.focus->client;
 
+    if(!sel)
+        return;
+
     for(c = globalconf.clients; !client_isvisible(c, screen); c = c->next);
     if(c == sel)
          for(sel = sel->next; sel && !client_isvisible(sel, screen); sel = sel->next);
 
-    if(!sel)
-        return;
-
-    client_detach(sel);
-    client_attach(sel);
-    arrange(screen);
+    if(sel)
+    {
+        client_list_detach(&globalconf.clients, sel);
+        client_list_push(&globalconf.clients, sel);
+        globalconf.screens[screen].need_arrange = True;
+    }
 }
 
+/** Send focus to next client in stack
+ * \param screen Screen ID
+ * \param arg Unused
+ * \ingroup ui_callback
+ */
+void
+uicb_client_focusnext(int screen, char *arg __attribute__ ((unused)))
+{
+    Client *c, *sel = globalconf.focus->client;
+
+    if(!sel)
+        return;
+    for(c = sel->next; c && (c->skip || !client_isvisible(c, screen)); c = c->next);
+    if(!c)
+        for(c = globalconf.clients; c && (c->skip || !client_isvisible(c, screen)); c = c->next);
+    if(c)
+        client_focus(c, screen, True);
+}
+
+/** Send focus to previous client in stack
+ * \param screen Screen ID
+ * \param arg Unused
+ * \ingroup ui_callback
+ */
+void
+uicb_client_focusprev(int screen, char *arg __attribute__ ((unused)))
+{
+    Client *prev;
+
+    if((prev = client_find_prev_visible(globalconf.focus->client)))
+        client_focus(prev, screen, True);
+}
+
+/** Toggle floating state of a client
+ * \param screen Screen ID
+ * \param arg unused
+ * \ingroup ui_callback
+ */
+void
+uicb_client_togglefloating(int screen __attribute__ ((unused)),
+                           char *arg __attribute__ ((unused)))
+{
+    if(globalconf.focus->client)
+        client_setfloating(globalconf.focus->client, !globalconf.focus->client->isfloating);
+}
+
+/** Toggle scratch client attribute
+ * \param screen screen number
+ * \param arg unused argument
+ * \ingroup ui_callback
+ */
+void
+uicb_client_setscratch(int screen,
+                       char *arg __attribute__ ((unused)))
+{
+    if(!globalconf.focus->client)
+        return;
+
+    if(globalconf.scratch.client == globalconf.focus->client)
+        globalconf.scratch.client = NULL;
+    else
+        globalconf.scratch.client = globalconf.focus->client;
+
+    widget_invalidate_cache(screen, WIDGET_CACHE_CLIENTS | WIDGET_CACHE_TAGS);
+    globalconf.screens[screen].need_arrange = True;
+}
+
+/** Toggle scratch client visibility
+ * \param screen screen number
+ * \param arg unused argument
+ * \ingroup ui_callback
+ */
+void
+uicb_client_togglescratch(int screen,
+                          char *arg __attribute__ ((unused)))
+{
+    if(globalconf.scratch.client)
+    {
+        globalconf.scratch.isvisible = !globalconf.scratch.isvisible;
+        if(globalconf.scratch.isvisible)
+            client_focus(globalconf.scratch.client, screen, True);
+        globalconf.screens[globalconf.scratch.client->screen].need_arrange = True;
+        widget_invalidate_cache(globalconf.scratch.client->screen, WIDGET_CACHE_CLIENTS);
+    }
+}
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
