@@ -22,34 +22,40 @@
 #ifndef AWESOME_CLIENT_H
 #define AWESOME_CLIENT_H
 
-#include "config.h"
+#include "structs.h"
 
 Bool client_isvisible(Client *, int);
-Client * get_client_bywin(Client *, Window);
-Client * get_client_byname(Client *, char *);
-void client_attach(Client *);
-void client_detach(Client *);
+Client * client_get_bywin(Client *, Window);
+Client * client_get_byname(Client *, char *);
+void client_focus(Client *, int, Bool);
 void client_ban(Client *);
-void focus(Client *, Bool, int);
+void client_unban(Client *);
 void client_manage(Window, XWindowAttributes *, int);
 Bool client_resize(Client *, Area, Bool);
-void client_unban(Client *);
 void client_unmanage(Client *);
 void client_updatewmhints(Client *);
-void client_updatesizehints(Client *);
+long client_updatesizehints(Client *);
 void client_updatetitle(Client *);
-void client_saveprops(Client *); 
+void client_saveprops(Client *);
 void client_kill(Client *);
+void client_setfloating(Client *, Bool);
 
 Uicb uicb_client_kill;
 Uicb uicb_client_moveresize;
 Uicb uicb_client_settrans;
 Uicb uicb_client_swapnext;
 Uicb uicb_client_swapprev;
-Uicb uicb_client_togglemax; 
-Uicb uicb_client_toggleverticalmax; 
-Uicb uicb_client_togglehorizontalmax; 
+Uicb uicb_client_togglemax;
+Uicb uicb_client_toggleverticalmax;
+Uicb uicb_client_togglehorizontalmax;
 Uicb uicb_client_zoom;
+Uicb uicb_client_focusnext;
+Uicb uicb_client_focusprev;
+Uicb uicb_client_togglefloating;
+Uicb uicb_client_togglescratch;
+Uicb uicb_client_setscratch;
+
+DO_SLIST(Client, client, p_delete);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
