@@ -24,25 +24,33 @@
 #include <X11/Xutil.h>
 #include <X11/extensions/shape.h>
 #include <X11/extensions/Xrandr.h>
+#include <X11/extensions/Xinerama.h>
 
 #include "screen.h"
 #include "event.h"
 #include "tag.h"
 #include "statusbar.h"
-#include "util.h"
 #include "window.h"
 #include "mouse.h"
 #include "ewmh.h"
 #include "client.h"
 #include "widget.h"
+#include "xutil.h"
+#include "rules.h"
 #include "layouts/tile.h"
 #include "layouts/floating.h"
-#include "xutil.h"
 
 extern AwesomeConf globalconf;
 
+/** Handle mouse button click
+ * \param screen screen number
+ * \param button button number
+ * \param state modkeys state
+ * \param buttons buttons list to check for
+ * \param arg optional arg passed to uicb, otherwise buttons' arg are used
+ */
 static void
-handle_mouse_button_press(int screen, unsigned int button, unsigned int state,
+event_handle_mouse_button_press(int screen, unsigned int button, unsigned int state,
                           Button *buttons, char *arg)
 {
     Button *b;
@@ -58,8 +66,11 @@ handle_mouse_button_press(int screen, unsigned int button, unsigned int state,
         }
 }
 
+/** Handle XButtonPressed events
+ * \param e XEvent
+ */
 void
-handle_event_buttonpress(XEvent *e)
+event_handle_buttonpress(XEvent *e)
 {
     int i, screen, x = 0, y = 0;
     unsigned int udummy;
@@ -69,9 +80,10 @@ handle_event_buttonpress(XEvent *e)
     Statusbar *statusbar;
     XButtonPressedEvent *ev = &e->xbutton;
 
-    for(screen = 0; screen < globalconf.nscreens; screen++)
+    for(screen = 0; screen < globalconf.nscreen; screen++)
         for(statusbar = globalconf.screens[screen].statusbar; statusbar; statusbar = statusbar->next)
-            if(statusbar->window == ev->window || statusbar->window == ev->subwindow)
+            if(statusbar->sw->window == ev->window || statusbar->sw->window == ev->subwindow)
+            {
                 switch(statusbar->position)
                 {
                   case Top:
@@ -87,37 +99,43 @@ handle_event_buttonpress(XEvent *e)
                   case Right:
                     for(widget = statusbar->widgets; widget; widget = widget->next)
                         if(ev->y >= widget->area.x && ev->y < widget->area.x + widget->area.width
-                           && statusbar->height - ev->x >= widget->area.y
-                           && statusbar->height - ev->x < widget->area.y + widget->area.height)
+                           && statusbar->sw->geometry.width - ev->x >= widget->area.y
+                           && statusbar->sw->geometry.width - ev->x
+                              < widget->area.y + widget->area.height)
                         {
                             widget->button_press(widget, ev);
                             return;
                         }
                     break;
-                 default:
+                  case Left:
                     for(widget = statusbar->widgets; widget; widget = widget->next)
-                        if(statusbar->width - ev->y >= widget->area.x
-                           && statusbar->width - ev->y < widget->area.x + widget->area.width
+                        if(statusbar->sw->geometry.height - ev->y >= widget->area.x
+                           && statusbar->sw->geometry.height - ev->y
+                              < widget->area.x + widget->area.width
                            && ev->x >= widget->area.y && ev->x < widget->area.y + widget->area.height)
                         {
                             widget->button_press(widget, ev);
                             return;
                         }
                     break;
+                  case Off:
+                    break;
                 }
+                /* return if no widget match */
+                return;
+            }
 
-    if((c = get_client_bywin(globalconf.clients, ev->window)))
+    if((c = client_get_bywin(globalconf.clients, ev->window)))
     {
-        focus(c, ev->same_screen, c->screen);
+        client_focus(c, c->screen, True);
         if(CLEANMASK(ev->state) == NoSymbol
            && ev->button == Button1)
         {
-            restack(c->screen);
             XAllowEvents(globalconf.display, ReplayPointer, CurrentTime);
-            window_grabbuttons(get_phys_screen(c->screen), c->win, True, True);
+            window_grabbuttons(get_phys_screen(c->screen), c->win);
         }
         else
-            handle_mouse_button_press(c->screen, ev->button, ev->state, globalconf.buttons.client, NULL);
+            event_handle_mouse_button_press(c->screen, ev->button, ev->state, globalconf.buttons.client, NULL);
     }
     else
         for(screen = 0; screen < ScreenCount(e->xany.display); screen++)
@@ -127,15 +145,18 @@ handle_event_buttonpress(XEvent *e)
                                 &wdummy, &x, &y, &i,
                                 &i, &udummy))
             {
-                screen = get_screen_bycoord(x, y);
-                handle_mouse_button_press(screen, ev->button, ev->state,
+                screen = screen_get_bycoord(screen, x, y);
+                event_handle_mouse_button_press(screen, ev->button, ev->state,
                                           globalconf.buttons.root, NULL);
                 return;
             }
 }
 
+/** Handle XConfigureRequest events
+ * \param e XEvent
+ */
 void
-handle_event_configurerequest(XEvent * e)
+event_handle_configurerequest(XEvent * e)
 {
     Client *c;
     XConfigureRequestEvent *ev = &e->xconfigurerequest;
@@ -143,7 +164,7 @@ handle_event_configurerequest(XEvent * e)
     int old_screen;
     Area geometry;
 
-    if((c = get_client_bywin(globalconf.clients, ev->window)))
+    if((c = client_get_bywin(globalconf.clients, ev->window)))
     {
         geometry = c->geometry;
 
@@ -163,15 +184,10 @@ handle_event_configurerequest(XEvent * e)
 
             client_resize(c, geometry, False);
 
-            tag_client_with_rules(c);
-            
-            if(old_screen != c->screen)
-            {
-                widget_invalidate_cache(old_screen, WIDGET_CACHE_CLIENTS);
-                arrange(old_screen);
-            }
-            widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
-            arrange(c->screen);
+            tag_client_with_rule(c, rule_matching_client(c));
+
+            if(!c->isfloating)
+                globalconf.screens[c->screen].need_arrange = True;
         }
         else
             window_configure(c->win, geometry, c->border);
@@ -189,137 +205,154 @@ handle_event_configurerequest(XEvent * e)
     }
 }
 
+/** Handle XConfigure events
+ * \param e XEvent
+ */
 void
-handle_event_configurenotify(XEvent * e)
+event_handle_configurenotify(XEvent * e)
 {
     XConfigureEvent *ev = &e->xconfigure;
     int screen;
-    Area area;
 
     for(screen = 0; screen < ScreenCount(e->xany.display); screen++)
         if(ev->window == RootWindow(e->xany.display, screen)
            && (ev->width != DisplayWidth(e->xany.display, screen)
                || ev->height != DisplayHeight(e->xany.display, screen)))
-        {
-            DisplayWidth(e->xany.display, screen) = ev->width;
-            DisplayHeight(e->xany.display, screen) = ev->height;
-
-            /* update statusbar */
-            area = get_screen_area(screen, NULL, &globalconf.screens[screen].padding);
-            globalconf.screens[screen].statusbar->width = area.width;
-
-            XResizeWindow(e->xany.display,
-                          globalconf.screens[screen].statusbar->window,
-                          globalconf.screens[screen].statusbar->width,
-                          globalconf.screens[screen].statusbar->height);
-
-            widget_invalidate_cache(screen, WIDGET_CACHE_ALL);
-            arrange(screen);
-        }
+            /* it's not that we panic, but restart */
+            uicb_exec(0, globalconf.argv);
 }
 
+/** Handle XDestroyWindow events
+ * \param e XEvent
+ */
 void
-handle_event_destroynotify(XEvent * e)
+event_handle_destroynotify(XEvent *e)
 {
     Client *c;
     XDestroyWindowEvent *ev = &e->xdestroywindow;
 
-    if((c = get_client_bywin(globalconf.clients, ev->window)))
+    if((c = client_get_bywin(globalconf.clients, ev->window)))
         client_unmanage(c);
 }
 
+/** Handle XCrossing events on enter
+ * \param e XEvent
+ */
 void
-handle_event_enternotify(XEvent * e)
+event_handle_enternotify(XEvent *e)
 {
     Client *c;
     XCrossingEvent *ev = &e->xcrossing;
     int screen;
-    Tag **curtags;
 
-    if(ev->mode != NotifyNormal)
+    if(ev->mode != NotifyNormal
+       || (ev->x_root == globalconf.pointer_x
+           && ev->y_root == globalconf.pointer_y))
         return;
-    if((c = get_client_bywin(globalconf.clients, ev->window)) && globalconf.screens[c->screen].sloppy_focus)
+
+    /* the idea behing saving pointer_x and pointer_y is Bob Marley powered */
+    globalconf.pointer_x = ev->x_root;
+    globalconf.pointer_y = ev->y_root;
+
+    if((c = client_get_bywin(globalconf.clients, ev->window)))
     {
-        focus(c, ev->same_screen, c->screen);
-        curtags = get_current_tags(c->screen);
-        if (c->isfloating || curtags[0]->layout->arrange == layout_floating)
-            window_grabbuttons(get_phys_screen(c->screen), c->win, True, False);
-        p_delete(&curtags);
+        window_grabbuttons(get_phys_screen(c->screen), c->win);
+        if(globalconf.screens[c->screen].sloppy_focus)
+            client_focus(c, c->screen,
+                         (globalconf.screens[c->screen].sloppy_focus
+                          && globalconf.screens[c->screen].sloppy_focus_raise));
     }
     else
-    {
         for(screen = 0; screen < ScreenCount(e->xany.display); screen++)
             if(ev->window == RootWindow(e->xany.display, screen))
-                focus(NULL, ev->same_screen, screen);
-        if((c = globalconf.focus->client))
-            window_grabbuttons(c->screen, c->win, False, False);
-    }
+                window_root_grabbuttons(screen);
+
 }
 
+/** Handle XMotion events
+ * \param e XEvent
+ */
 void
-handle_event_expose(XEvent *e)
+event_handle_motionnotify(XEvent *e)
+{
+    XMotionEvent *ev = &e->xmotion;
+
+    globalconf.pointer_x = ev->x_root;
+    globalconf.pointer_y = ev->y_root;
+}
+
+/** Handle XExpose events
+ * \param e XEvent
+ */
+void
+event_handle_expose(XEvent *e)
 {
     XExposeEvent *ev = &e->xexpose;
     int screen;
     Statusbar *statusbar;
 
     if(!ev->count)
-        for(screen = 0; screen < globalconf.nscreens; screen++)
+        for(screen = 0; screen < globalconf.nscreen; screen++)
             for(statusbar = globalconf.screens[screen].statusbar; statusbar; statusbar = statusbar->next)
-                if(statusbar->window == ev->window)
+                if(statusbar->sw->window == ev->window)
                 {
                     statusbar_display(statusbar);
                     return;
                 }
 }
 
+/** Handle XKey events
+ * \param e XEvent
+ */
 void
-handle_event_keypress(XEvent * e)
+event_handle_keypress(XEvent *e)
 {
     int screen, x, y, d;
     unsigned int m;
-    KeySym keysym;
     XKeyEvent *ev = &e->xkey;
     Window dummy;
     Key *k;
 
-    keysym = XKeycodeToKeysym(e->xany.display, (KeyCode) ev->keycode, 0);
-
     /* find the right screen for this event */
     for(screen = 0; screen < ScreenCount(e->xany.display); screen++)
         if(XQueryPointer(e->xany.display, RootWindow(e->xany.display, screen), &dummy, &dummy, &x, &y, &d, &d, &m))
         {
             /* if screen is 0, we are on first Zaphod screen or on the
              * only screen in Xinerama, so we can ask for a better screen
-             * number with get_screen_bycoord: we'll get 0 in Zaphod mode
+             * number with screen_get_bycoord: we'll get 0 in Zaphod mode
              * so it's the same, or maybe the real Xinerama screen */
-            if(screen == 0)
-                screen = get_screen_bycoord(x, y);
+            screen = screen_get_bycoord(screen, x, y);
             break;
         }
 
     for(k = globalconf.keys; k; k = k->next)
-        if(keysym == k->keysym && k->func
-           && CLEANMASK(k->mod) == CLEANMASK(ev->state))
+        if(ev->keycode == k->keycode &&
+	  k->func && CLEANMASK(k->mod) == CLEANMASK(ev->state))
         {
             k->func(screen, k->arg);
             break;
         }
 }
 
+/** Handle XCrossing events on leave
+ * \param e XEvent
+ */
 void
-handle_event_leavenotify(XEvent * e)
+event_handle_leavenotify(XEvent * e)
 {
     XCrossingEvent *ev = &e->xcrossing;
     int screen;
 
     for(screen = 0; screen < ScreenCount(e->xany.display); screen++)
         if((ev->window == RootWindow(e->xany.display, screen)) && !ev->same_screen)
-            focus(NULL, ev->same_screen, screen);
+            client_focus(NULL, screen, True);
 }
 
+/** Handle XMapping events
+ * \param e XEvent
+ */
 void
-handle_event_mappingnotify(XEvent *e)
+event_handle_mappingnotify(XEvent *e)
 {
     XMappingEvent *ev = &e->xmapping;
     int screen;
@@ -327,37 +360,43 @@ handle_event_mappingnotify(XEvent *e)
     XRefreshKeyboardMapping(ev);
     if(ev->request == MappingKeyboard)
         for(screen = 0; screen < ScreenCount(e->xany.display); screen++)
-            grabkeys(get_phys_screen(screen));
+            window_root_grabkeys(screen);
 }
 
+/** Handle XMapRequest events
+ * \param e XEvent
+ */
 void
-handle_event_maprequest(XEvent *e)
+event_handle_maprequest(XEvent *e)
 {
     static XWindowAttributes wa;
     XMapRequestEvent *ev = &e->xmaprequest;
-    int screen, x, y, d;
+    int screen = 0, x, y, d;
     unsigned int m;
     Window dummy;
-    XEvent event;
 
     if(!XGetWindowAttributes(e->xany.display, ev->window, &wa))
         return;
     if(wa.override_redirect)
         return;
-    if(!get_client_bywin(globalconf.clients, ev->window))
+    if(!client_get_bywin(globalconf.clients, ev->window))
     {
-        for(screen = 0; wa.screen != ScreenOfDisplay(e->xany.display, screen); screen++);
-        if(screen == 0 && XQueryPointer(e->xany.display, RootWindow(e->xany.display, screen),
-                                        &dummy, &dummy, &x, &y, &d, &d, &m))
-            screen = get_screen_bycoord(x, y);
+        if(XineramaIsActive(globalconf.display)
+           && XQueryPointer(e->xany.display, RootWindow(e->xany.display, screen),
+                            &dummy, &dummy, &x, &y, &d, &d, &m))
+            screen = screen_get_bycoord(screen, x, y);
+        else
+             for(screen = 0; wa.screen != ScreenOfDisplay(e->xany.display, screen); screen++);
+
         client_manage(ev->window, &wa, screen);
-        /* do this to keep focused client */
-        while(XCheckMaskEvent(globalconf.display, EnterWindowMask, &event));
     }
 }
 
+/** Handle XProperty events
+ * \param e XEvent
+ */
 void
-handle_event_propertynotify(XEvent * e)
+event_handle_propertynotify(XEvent * e)
 {
     Client *c;
     Window trans;
@@ -365,14 +404,15 @@ handle_event_propertynotify(XEvent * e)
 
     if(ev->state == PropertyDelete)
         return;                 /* ignore */
-    if((c = get_client_bywin(globalconf.clients, ev->window)))
+    if((c = client_get_bywin(globalconf.clients, ev->window)))
     {
         switch (ev->atom)
         {
           case XA_WM_TRANSIENT_FOR:
             XGetTransientForHint(e->xany.display, c->win, &trans);
-            if(!c->isfloating && (c->isfloating = (get_client_bywin(globalconf.clients, trans) != NULL)))
-                arrange(c->screen);
+            if(!c->isfloating
+               && (c->isfloating = (client_get_bywin(globalconf.clients, trans) != NULL)))
+                globalconf.screens[c->screen].need_arrange = True;
             break;
           case XA_WM_NORMAL_HINTS:
             client_updatesizehints(c);
@@ -386,59 +426,51 @@ handle_event_propertynotify(XEvent * e)
     }
 }
 
+/** Handle XUnmap events
+ * \param e XEvent
+ */
 void
-handle_event_unmapnotify(XEvent * e)
+event_handle_unmapnotify(XEvent * e)
 {
     Client *c;
     XUnmapEvent *ev = &e->xunmap;
-    XEvent event;
 
-    if((c = get_client_bywin(globalconf.clients, ev->window))
+    if((c = client_get_bywin(globalconf.clients, ev->window))
        && ev->event == RootWindow(e->xany.display, get_phys_screen(c->screen))
        && ev->send_event && window_getstate(c->win) == NormalState)
         client_unmanage(c);
-    /* do this to keep focused client */
-    while(XCheckMaskEvent(globalconf.display, EnterWindowMask, &event));
 }
 
+/** Handle XShape events
+ * \param e XEvent
+ */
 void
-handle_event_shape(XEvent * e)
+event_handle_shape(XEvent * e)
 {
     XShapeEvent *ev = (XShapeEvent *) e;
-    Client *c = get_client_bywin(globalconf.clients, ev->window);
+    Client *c = client_get_bywin(globalconf.clients, ev->window);
 
     if(c)
         window_setshape(get_phys_screen(c->screen), c->win);
 }
 
+/** Handle XRandR events
+ * \param e XEvent
+ */
 void
-handle_event_randr_screen_change_notify(XEvent *e)
+event_handle_randr_screen_change_notify(XEvent *e)
 {
     XRRUpdateConfiguration(e);
     uicb_exec(0, globalconf.argv);
 }
 
+/** Handle XClientMessage events
+ * \param e XEvent
+ */
 void
-handle_event_clientmessage(XEvent *e)
+event_handle_clientmessage(XEvent *e)
 {
     ewmh_process_client_message(&e->xclient);
 }
 
-void
-grabkeys(int phys_screen)
-{
-    Key *k;
-    KeyCode code;
-
-    XUngrabKey(globalconf.display, AnyKey, AnyModifier, RootWindow(globalconf.display, phys_screen));
-    for(k = globalconf.keys; k; k = k->next)
-    {
-        if((code = XKeysymToKeycode(globalconf.display, k->keysym)) == NoSymbol)
-            continue;
-        XGrabKey(globalconf.display, code, k->mod, RootWindow(globalconf.display, phys_screen), True, GrabModeAsync, GrabModeAsync);
-        XGrabKey(globalconf.display, code, k->mod | LockMask, RootWindow(globalconf.display, phys_screen), True, GrabModeAsync, GrabModeAsync);
-        XGrabKey(globalconf.display, code, k->mod | globalconf.numlockmask, RootWindow(globalconf.display, phys_screen), True, GrabModeAsync, GrabModeAsync);
-        XGrabKey(globalconf.display, code, k->mod | globalconf.numlockmask | LockMask, RootWindow(globalconf.display, phys_screen), True, GrabModeAsync, GrabModeAsync);
-    }
-}
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
