@@ -22,7 +22,11 @@
 #ifndef AWESOME_LAYOUT_H
 #define AWESOME_LAYOUT_H
 
+#include <X11/Xlib.h>
+
 #include "uicb.h"
+#include "common/list.h"
+#include "common/util.h"
 
 typedef void (LayoutArrange)(int);
 
@@ -31,20 +35,15 @@ struct Layout
 {
     char *image;
     LayoutArrange *arrange;
-    Layout *prev;
     Layout *next;
 };
 
-void arrange(int);
-Layout * get_current_layout(int);
-void restack(int);
-void loadawesomeprops(int);
-void saveawesomeprops(int);
+DO_SLIST(Layout, layout, p_delete);
+
+int layout_refresh(void);
+Layout * layout_get_current(int);
 
-Uicb uicb_client_focusnext;
-Uicb uicb_client_focusprev;
 Uicb uicb_tag_setlayout;
-Uicb uicb_client_togglefloating;
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
