@@ -22,14 +22,16 @@
 #ifndef AWESOME_RULES_H
 #define AWESOME_RULES_H
 
-#include "config.h"
+#include "structs.h"
 
 #define RULE_NOSCREEN        -1
 
-regex_t * rules_compile_regex(char *);    
-Bool client_match_rule(Client *, Rule *);
-Bool is_tag_match_rules(Tag *, Rule *);
-RuleFloat rules_get_float_from_str(const char *);
+regex_t * rules_compile_regex(char *);
+Bool tag_match_rule(Tag *, Rule *);
+Fuzzy rules_get_fuzzy_from_str(const char *);
+Rule * rule_matching_client(Client *);
+
+DO_SLIST(Rule, rule, p_delete);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
