@@ -1,7 +1,7 @@
 /*
  * screen.c - screen management
  *
- * Copyright © 2007 Julien Danjou <julien@danjou.info>
+ * Copyright © 2007-2008 Julien Danjou <julien@danjou.info>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 #include <X11/extensions/Xinerama.h>
 
-#include "util.h"
 #include "screen.h"
 #include "tag.h"
 #include "focus.h"
@@ -37,34 +36,14 @@ extern AwesomeConf globalconf;
  * \return Area
  */
 Area
-get_screen_area(int screen, Statusbar *statusbar, Padding *padding)
+screen_get_area(int screen, Statusbar *statusbar, Padding *padding)
 {
-    int screen_number = 0;
-    XineramaScreenInfo *si;
     Area area;
     Statusbar *sb;
 
-    if(XineramaIsActive(globalconf.display))
-    {
-        si = XineramaQueryScreens(globalconf.display, &screen_number);
-        if (screen_number < screen)
-            eprint("Info request for unknown screen.");
-        area.x = si[screen].x_org;
-        area.y = si[screen].y_org;
-        area.width = si[screen].width;
-        area.height = si[screen].height;
-        XFree(si);
-    }
-    else
-    {
-        /* emulate Xinerama info but only fill the screen we want */
-        area.x = 0;
-        area.y = 0;
-        area.width = DisplayWidth(globalconf.display, screen);
-        area.height = DisplayHeight(globalconf.display, screen);
-    }
+    area = globalconf.screens[screen].geometry;
 
-     /* make padding corrections */
+    /* make padding corrections */
     if(padding)
     {
         area.x += padding->left;
@@ -102,11 +81,9 @@ get_screen_area(int screen, Statusbar *statusbar, Padding *padding)
 Area
 get_display_area(int screen, Statusbar *statusbar, Padding *padding)
 {
-    Area area;
+    Area area = { 0, 0, 0, 0, NULL };
     Statusbar *sb;
 
-    area.x = 0;
-    area.y = 0;
     area.width = DisplayWidth(globalconf.display, screen);
     area.height = DisplayHeight(globalconf.display, screen);
 
@@ -133,18 +110,18 @@ get_display_area(int screen, Statusbar *statusbar, Padding *padding)
  * \return screen number or DefaultScreen of disp on no match
  */
 int
-get_screen_bycoord(int x, int y)
+screen_get_bycoord(int screen, int x, int y)
 {
     int i;
     Area area;
 
     /* don't waste our time */
     if(!XineramaIsActive(globalconf.display))
-        return DefaultScreen(globalconf.display);
+        return screen;
 
-    for(i = 0; i < globalconf.nscreens; i++)
+    for(i = 0; i < globalconf.nscreen; i++)
     {
-        area = get_screen_area(i, NULL, NULL);
+        area = screen_get_area(i, NULL, NULL);
         if((x < 0 || (x >= area.x && x < area.x + area.width))
            && (y < 0 || (y >= area.y && y < area.y + area.height)))
             return i;
@@ -152,20 +129,80 @@ get_screen_bycoord(int x, int y)
     return DefaultScreen(globalconf.display);
 }
 
-/** Return the actual screen count
- * \return the number of screen available
- */
-int
-get_screen_count(void)
+
+static inline Area
+screen_xsi_to_area(XineramaScreenInfo si)
 {
-    int screen_number;
+    Area a;
+
+    a.x = si.x_org;
+    a.y = si.y_org;
+    a.width = si.width;
+    a.height = si.height;
+    a.next = NULL;
+
+    return a;
+}
+
+void
+screen_build_screens(void)
+{
+    XineramaScreenInfo *si;
+    int xinerama_screen_number, screen, screen_to_test;
+    Bool drop;
 
     if(XineramaIsActive(globalconf.display))
-        XFree(XineramaQueryScreens(globalconf.display, &screen_number));
+    {
+        si = XineramaQueryScreens(globalconf.display, &xinerama_screen_number);
+        globalconf.screens = p_new(VirtScreen, xinerama_screen_number);
+        globalconf.nscreen = 0;
+
+        /* now check if screens overlaps (same x,y): if so, we take only the biggest one */
+        for(screen = 0; screen < xinerama_screen_number; screen++)
+        {
+            drop = False;
+            for(screen_to_test = 0; screen_to_test < globalconf.nscreen; screen_to_test++)
+                if(si[screen].x_org == globalconf.screens[screen_to_test].geometry.x
+                   && si[screen].y_org == globalconf.screens[screen_to_test].geometry.y)
+                    {
+                        /* we already have a screen for this area, just check if
+                         * it's not bigger and drop it */
+                        drop = True;
+                        globalconf.screens[screen_to_test].geometry.width =
+                            MAX(si[screen].width, si[screen_to_test].width);
+                        globalconf.screens[screen_to_test].geometry.height =
+                            MAX(si[screen].height, si[screen_to_test].height);
+                    }
+            if(!drop)
+                globalconf.screens[globalconf.nscreen++].geometry = screen_xsi_to_area(si[screen]);
+        }
+
+        /* realloc smaller if xinerama_screen_number != screen registered */
+        if(xinerama_screen_number != globalconf.nscreen)
+        {
+            VirtScreen *newscreens = p_new(VirtScreen, globalconf.nscreen);
+            memcpy(newscreens, globalconf.screens, globalconf.nscreen * sizeof(VirtScreen));
+            p_delete(&globalconf.screens);
+            globalconf.screens = newscreens;
+        }
+
+        XFree(si);
+    }
     else
-        screen_number = ScreenCount(globalconf.display);
+    {
+        globalconf.nscreen = ScreenCount(globalconf.display);
+        globalconf.screens = p_new(VirtScreen, globalconf.nscreen);
+        for(screen = 0; screen < globalconf.nscreen; screen++)
+        {
+            globalconf.screens[screen].geometry.x = 0;
+            globalconf.screens[screen].geometry.y = 0;
+            globalconf.screens[screen].geometry.width =
+                DisplayWidth(globalconf.display, screen);
+            globalconf.screens[screen].geometry.height =
+                DisplayHeight(globalconf.display, screen);
+        }
+    }
 
-    return screen_number;
 }
 
 /** This returns the real X screen number for a logical
@@ -208,8 +245,8 @@ move_client_to_screen(Client *c, int new_screen, Bool doresize)
         Area new_geometry, new_f_geometry;
         new_f_geometry = c->f_geometry;
 
-        to = get_screen_area(c->screen, NULL, NULL);
-        from = get_screen_area(old_screen, NULL, NULL);
+        to = screen_get_area(c->screen, NULL, NULL);
+        from = screen_get_area(old_screen, NULL, NULL);
 
         /* compute new coords in new screen */
         new_f_geometry.x = (c->f_geometry.x - from.x) + to.x;
@@ -266,7 +303,8 @@ move_client_to_screen(Client *c, int new_screen, Bool doresize)
         else
         {
             c->f_geometry = new_f_geometry;
-            arrange(c->screen);
+            globalconf.screens[old_screen].need_arrange = True;
+            globalconf.screens[c->screen].need_arrange = True;
         }
     }
 }
@@ -275,11 +313,11 @@ move_client_to_screen(Client *c, int new_screen, Bool doresize)
  * \param screen screen number
  */
 static void
-move_mouse_pointer_to_screen(int screen)
+move_mouse_pointer_to_screen(int phys_screen)
 {
     if(XineramaIsActive(globalconf.display))
     {
-        Area area = get_screen_area(screen, NULL, NULL);
+        Area area = screen_get_area(phys_screen, NULL, NULL);
         XWarpPointer(globalconf.display,
                      None,
                      DefaultRootWindow(globalconf.display),
@@ -288,7 +326,7 @@ move_mouse_pointer_to_screen(int screen)
     else
         XWarpPointer(globalconf.display,
                      None,
-                     RootWindow(globalconf.display, screen),
+                     RootWindow(globalconf.display, phys_screen),
                      0, 0, 0, 0, 0, 0);
 }
 
@@ -309,11 +347,11 @@ uicb_screen_focus(int screen, char *arg)
         new_screen = screen + 1;
 
     if (new_screen < 0)
-        new_screen = globalconf.nscreens - 1;
-    if (new_screen > (globalconf.nscreens - 1))
+        new_screen = globalconf.nscreen - 1;
+    if (new_screen > (globalconf.nscreen - 1))
         new_screen = 0;
 
-    focus(focus_get_current_client(new_screen), True, new_screen);
+    client_focus(NULL, new_screen, True);
 
     move_mouse_pointer_to_screen(new_screen);
 }
@@ -328,7 +366,6 @@ uicb_client_movetoscreen(int screen __attribute__ ((unused)), char *arg)
 {
     int new_screen, prev_screen;
     Client *sel = globalconf.focus->client;
-    XEvent event;
 
     if(!sel || !XineramaIsActive(globalconf.display))
         return;
@@ -338,18 +375,14 @@ uicb_client_movetoscreen(int screen __attribute__ ((unused)), char *arg)
     else
         new_screen = sel->screen + 1;
 
-    if(new_screen >= globalconf.nscreens)
+    if(new_screen >= globalconf.nscreen)
         new_screen = 0;
     else if(new_screen < 0)
-        new_screen = globalconf.nscreens - 1;
+        new_screen = globalconf.nscreen - 1;
 
     prev_screen = sel->screen;
     move_client_to_screen(sel, new_screen, True);
     move_mouse_pointer_to_screen(new_screen);
-    arrange(prev_screen);
-    arrange(new_screen);
-    focus(sel, True, sel->screen);
-    /* drop EnterWindow event to keep focus */
-    XCheckMaskEvent(globalconf.display, EnterWindowMask, &event);
+    client_focus(sel, sel->screen, True);
 }
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
