@@ -22,15 +22,17 @@
 #ifndef AWESOME_STATUSBAR_H
 #define AWESOME_STATUSBAR_H
 
-#include "config.h"
+#include "structs.h"
 
 void statusbar_refresh(void);
-void statusbar_draw(Statusbar *);
-void statusbar_init(Statusbar *, int);
+void statusbar_preinit(Statusbar *);
+void statusbar_init(Statusbar *);
 void statusbar_display(Statusbar *);
 Position statusbar_get_position_from_str(const char *);
 
 Uicb uicb_statusbar_toggle;
 
+DO_SLIST(Statusbar, statusbar, p_delete);
+
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
