@@ -24,7 +24,6 @@
 
 #include "screen.h"
 #include "tag.h"
-#include "util.h"
 #include "rules.h"
 #include "client.h"
 #include "ewmh.h"
@@ -32,66 +31,85 @@
 
 extern AwesomeConf globalconf;
 
-static void
-detach_tagclientlink(TagClientLink *tc)
+Tag *
+tag_new(const char *name, Layout *layout, double mwfact, int nmaster, int ncol)
 {
-    TagClientLink *tmp;
+    Tag *tag;
 
-    if(globalconf.tclink == tc)
-        globalconf.tclink = tc->next;
-    else
-    {
-        for(tmp = globalconf.tclink; tmp && tmp->next != tc; tmp = tmp->next);
-        tmp->next = tc->next;
-    }
-    
-    p_delete(&tc);
+    tag = p_new(Tag, 1);
+    tag->name = a_strdup(name);
+    tag->layout = layout;
+
+    tag->mwfact = mwfact;
+    if(tag->mwfact <= 0 || tag->mwfact >= 1)
+        tag->mwfact = 0.5;
+
+    if((tag->nmaster = nmaster) < 0)
+        tag->nmaster = 1;
+
+    if((tag->ncol = ncol) < 1)
+        tag->ncol = 1;
+
+    return tag;
+}
+
+static void
+tag_append_to_screen(Tag *tag, int screen)
+{
+    tag->screen = screen;
+    tag_list_append(&globalconf.screens[screen].tags, tag);
+    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
+}
+
+void
+tag_push_to_screen(Tag *tag, int screen)
+{
+    tag->screen = screen;
+    tag_list_push(&globalconf.screens[screen].tags, tag);
+    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
 }
 
 void
 tag_client(Client *c, Tag *t)
 {
-    TagClientLink *tc, *new_tc;
+    tag_client_node_t *tc;
 
     /* don't tag twice */
     if(is_client_tagged(c, t))
         return;
 
-    new_tc = p_new(TagClientLink, 1);
-
-    if(!globalconf.tclink)
-        globalconf.tclink = new_tc;
-    else
-    {
-        for(tc = globalconf.tclink; tc->next; tc = tc->next);
-        tc->next = new_tc;
-    }
-
-    new_tc->client = c;
-    new_tc->tag = t;
+    tc = p_new(tag_client_node_t, 1);
+    tc->client = c;
+    tc->tag = t;
+    tag_client_node_list_push(&globalconf.tclink, tc);
 
+    client_saveprops(c);
     widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
+    globalconf.screens[c->screen].need_arrange = True;
 }
 
 void
 untag_client(Client *c, Tag *t)
 {
-    TagClientLink *tc;
+    tag_client_node_t *tc;
 
     for(tc = globalconf.tclink; tc; tc = tc->next)
         if(tc->client == c && tc->tag == t)
         {
-            detach_tagclientlink(tc);
+            tag_client_node_list_detach(&globalconf.tclink, tc);
+            p_delete(&tc);
             break;
         }
 
+    client_saveprops(c);
     widget_invalidate_cache(c->screen, WIDGET_CACHE_CLIENTS);
+    globalconf.screens[c->screen].need_arrange = True;
 }
 
 Bool
 is_client_tagged(Client *c, Tag *t)
 {
-    TagClientLink *tc;
+    tag_client_node_t *tc;
 
     if(!c)
         return False;
@@ -117,49 +135,31 @@ tag_client_with_current_selected(Client *c)
 }
 
 void
-tag_client_with_rules(Client *c)
+tag_client_with_rule(Client *c, Rule *r)
 {
-    Rule *r;
     Tag *tag;
     Bool matched = False;
 
-    for(r = globalconf.rules; r; r = r->next)
-        if(client_match_rule(c, r))
+    if(!r) return;
+
+    /* check if at least one tag match */
+    for(tag = globalconf.screens[c->screen].tags; tag; tag = tag->next)
+        if(tag_match_rule(tag, r))
         {
-            switch(r->isfloating)
-            {
-              case Tile:
-                c->isfloating = False;
-                break;
-              case Float:
-                c->isfloating = True;
-                client_resize(c, c->f_geometry, False);
-                break;
-              default:
-                break;
-            }
-
-            if(r->screen != RULE_NOSCREEN && r->screen != c->screen)
-                move_client_to_screen(c, r->screen, True);
-
-            for(tag = globalconf.screens[c->screen].tags; tag; tag = tag->next)
-                if(is_tag_match_rules(tag, r))
-                {
-                    matched = True;
-                    tag_client(c, tag);
-                }
-                else
-                    untag_client(c, tag);
-
-            if(!matched)
-                tag_client_with_current_selected(c);
+            matched = True;
             break;
         }
+
+    if(matched)
+        for(tag = globalconf.screens[c->screen].tags; tag; tag = tag->next)
+            if(tag_match_rule(tag, r))
+                tag_client(c, tag);
+            else
+                untag_client(c, tag);
 }
 
-
 Tag **
-get_current_tags(int screen)
+tags_get_current(int screen)
 {
     Tag *tag, **tags = NULL;
     int n = 1;
@@ -211,9 +211,6 @@ uicb_client_tag(int screen, char *arg)
     else
         for(tag = globalconf.screens[screen].tags; tag; tag = tag->next)
             tag_client(sel, tag);
-
-    client_saveprops(sel);
-    arrange(screen);
 }
 
 /** Toggle a tag on client
@@ -246,7 +243,7 @@ uicb_client_toggletag(int screen, char *arg)
 
         /* check that there's at least one tag selected for this client*/
         for(tag = globalconf.screens[screen].tags; tag
-            && !is_client_tagged(sel, tag); tag = tag->next)
+            && !is_client_tagged(sel, tag); tag = tag->next);
 
         if(!tag)
             tag_client(sel, target_tag);
@@ -257,9 +254,6 @@ uicb_client_toggletag(int screen, char *arg)
                 tag_client(sel, tag);
             else
                 untag_client(sel, tag);
-
-    client_saveprops(sel);
-    arrange(screen);
 }
 
 /** Add a tag to viewed tags
@@ -280,43 +274,63 @@ uicb_tag_toggleview(int screen, char *arg)
             target_tag = target_tag->next, i--);
 
         if(target_tag)
-            target_tag->selected = !target_tag->selected;
+            tag_view(target_tag, !target_tag->selected);
 
         /* check that there's at least one tag selected */
         for(tag = globalconf.screens[screen].tags; tag && !tag->selected; tag = tag->next);
         if(!tag)
-            target_tag->selected = True;
+            tag_view(target_tag, True);
     }
     else
         for(tag = globalconf.screens[screen].tags; tag; tag = tag->next)
-            tag->selected = !tag->selected;
+            tag_view(tag, !tag->selected);
+}
 
-    saveawesomeprops(screen);
-    arrange(screen);
-    ewmh_update_net_current_desktop(get_phys_screen(screen));
-    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
+static void
+tag_view_only(Tag *target)
+{
+    Tag *tag;
+
+    if(!target) return;
+
+    for(tag = globalconf.screens[target->screen].tags; tag; tag = tag->next)
+        tag_view(tag, tag == target);
 }
 
 void
-tag_view(int screen, int dindex)
+tag_view_byindex(int screen, int dindex, Bool view)
 {
-    Tag *target_tag, *tag;
+    Tag *tag;
 
     if(dindex < 0)
         return;
 
-    for(target_tag = globalconf.screens[screen].tags; target_tag && dindex > 0;
-        target_tag = target_tag->next, dindex--);
-    if(target_tag)
-    {
-        for(tag = globalconf.screens[screen].tags; tag; tag = tag->next)
-            tag->selected = False;
-        target_tag->selected = True;
-    }
-    saveawesomeprops(screen);
-    arrange(screen);
-    ewmh_update_net_current_desktop(get_phys_screen(screen));
-    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
+    for(tag = globalconf.screens[screen].tags; tag && dindex > 0;
+        tag = tag->next, dindex--);
+    tag_view(tag, view);
+}
+
+void
+tag_view_only_byindex(int screen, int dindex)
+{
+    Tag *tag;
+
+    if(dindex < 0)
+        return;
+
+    for(tag = globalconf.screens[screen].tags; tag && dindex > 0;
+        tag = tag->next, dindex--);
+    tag_view_only(tag);
+}
+
+void
+tag_view(Tag *tag, Bool view)
+{
+    tag->was_selected = tag->selected;
+    tag->selected = view;
+    ewmh_update_net_current_desktop(get_phys_screen(tag->screen));
+    widget_invalidate_cache(tag->screen, WIDGET_CACHE_TAGS);
+    globalconf.screens[tag->screen].need_arrange = True;
 }
 
 /** View tag
@@ -330,16 +344,10 @@ uicb_tag_view(int screen, char *arg)
     Tag *tag;
 
     if(arg)
-	tag_view(screen, atoi(arg) - 1);
+	tag_view_only_byindex(screen, atoi(arg) - 1);
     else
-    {
         for(tag = globalconf.screens[screen].tags; tag; tag = tag->next)
-            tag->selected = True;
-        saveawesomeprops(screen);
-        arrange(screen);
-        ewmh_update_net_current_desktop(get_phys_screen(screen));
-        widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
-    }
+            tag_view(tag, True);
 }
 
 /** View previously selected tags
@@ -351,17 +359,9 @@ void
 uicb_tag_prev_selected(int screen, char *arg __attribute__ ((unused)))
 {
     Tag *tag;
-    Bool t;
 
-    for(tag =  globalconf.screens[screen].tags; tag; tag = tag->next)
-    {
-        t = tag->selected;
-        tag->selected = tag->was_selected;
-        tag->was_selected = t;
-    }
-    arrange(screen);
-    ewmh_update_net_current_desktop(get_phys_screen(screen));
-    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
+    for(tag = globalconf.screens[screen].tags; tag; tag = tag->next)
+        tag_view(tag, tag->was_selected);
 }
 
 /** View next tag
@@ -372,20 +372,14 @@ uicb_tag_prev_selected(int screen, char *arg __attribute__ ((unused)))
 void
 uicb_tag_viewnext(int screen, char *arg __attribute__ ((unused)))
 {
-    Tag **curtags = get_current_tags(screen);
+    Tag *tag, **curtags = tags_get_current(screen);
 
-    if(!curtags[0]->next)
-        return;
+    tag = tag_list_next_cycle(&globalconf.screens[screen].tags, curtags[0]);
 
-    curtags[0]->selected = False;
-    curtags[0]->next->selected = True;
+    tag_view(curtags[0], False);
+    tag_view(tag, True);
 
     p_delete(&curtags);
-
-    saveawesomeprops(screen);
-    arrange(screen);
-    ewmh_update_net_current_desktop(get_phys_screen(screen));
-    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
 }
 
 /** View previous tag
@@ -396,37 +390,26 @@ uicb_tag_viewnext(int screen, char *arg __attribute__ ((unused)))
 void
 uicb_tag_viewprev(int screen, char *arg __attribute__ ((unused)))
 {
-    Tag *tag, **curtags = get_current_tags(screen);
+    Tag *tag, **curtags = tags_get_current(screen);
+
+    tag = tag_list_prev_cycle(&globalconf.screens[screen].tags, curtags[0]);
+
+    tag_view(curtags[0], False);
+    tag_view(tag, True);
 
-    for(tag = globalconf.screens[screen].tags; tag && tag->next != curtags[0]; tag = tag->next);
-    if(tag)
-    {
-        tag->selected = True;
-        curtags[0]->selected = False;
-        saveawesomeprops(screen);
-        arrange(screen);
-    }
     p_delete(&curtags);
-    ewmh_update_net_current_desktop(get_phys_screen(screen));
-    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
 }
 
 void
 uicb_tag_create(int screen, char *arg)
 {
-    Tag *last_tag, *tag;
+    Tag *tag;
 
     if(!a_strlen(arg))
         return;
 
-    for(last_tag = globalconf.screens[screen].tags; last_tag && last_tag->next; last_tag = last_tag->next);
-    last_tag->next = tag = p_new(Tag, 1);
-    tag->name = a_strdup(arg);
-    tag->layout = globalconf.screens[screen].layouts;
-    tag->mwfact = 0.5;
-    tag->nmaster = 1;
-    tag->ncol = 1;
-    widget_invalidate_cache(screen, WIDGET_CACHE_TAGS);
+    tag = tag_new(arg, globalconf.screens[screen].layouts, 0.5, 1, 1);
+    tag_append_to_screen(tag, screen);
 }
 
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
