@@ -22,18 +22,22 @@
 #ifndef AWESOME_TAG_H
 #define AWESOME_TAG_H
 
-#include "config.h"
+#include "structs.h"
 
 /** Check if a client is tiled */
-#define IS_TILED(client, screen)            (client && !client->isfloating && client_isvisible(client, screen))
+#define IS_TILED(client, screen)            (client && !client->isfloating && !client->ismax && client_isvisible(client, screen))
 
-Tag ** get_current_tags(int );
+Tag * tag_new(const char *, Layout *, double, int, int);
+void tag_view(Tag *, Bool);
+void tag_view_byindex(int, int, Bool);
+void tag_push_to_screen(Tag *, int);
+Tag ** tags_get_current(int);
 void tag_client(Client *, Tag *);
 void untag_client(Client *, Tag *);
 Bool is_client_tagged(Client *, Tag *);
+void tag_client_with_rule(Client *, Rule *r);
 void tag_client_with_current_selected(Client *);
-void tag_client_with_rules(Client *);
-void tag_view(int, int);
+void tag_view_only_byindex(int, int);
 
 Uicb uicb_client_tag;
 Uicb uicb_client_toggletag;
@@ -44,5 +48,8 @@ Uicb uicb_tag_viewnext;
 Uicb uicb_tag_viewprev;
 Uicb uicb_tag_create;
 
+DO_SLIST(Tag, tag, p_delete);
+DO_SLIST(tag_client_node_t, tag_client_node, p_delete);
+
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
