@@ -1,7 +1,7 @@
 /*
  * window.h - window handling functions header
  *
- * Copyright © 2007 Julien Danjou <julien@danjou.info>
+ * Copyright © 2007-2008 Julien Danjou <julien@danjou.info>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,17 +22,16 @@
 #ifndef AWESOME_WINDOW_H
 #define AWESOME_WINDOW_H
 
-#include <X11/Xlib.h>
-
-/** Mask shorthands, used in event.c and window.c */ 
-#define BUTTONMASK              (ButtonPressMask | ButtonReleaseMask) 
+#include "common/draw.h"
 
 int window_setstate(Window, long);
 long window_getstate(Window);
 Status window_configure(Window, Area, int);
-void window_grabbuttons(int, Window, Bool, Bool);
+void window_grabbuttons(int, Window);
+void window_root_grabbuttons(int);
+void window_root_grabkeys(int);
 void window_setshape(int, Window);
-void window_settrans(Window, double);
+int window_settrans(Window, double);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
